@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Stéphane Tayeb, Aymeric PELLE
@@ -28,13 +29,13 @@
 
 #include <CGAL/license/Mesh_3.h>
 
+#include <CGAL/disable_warnings.h>
 
 #include <CGAL/Mesh_3/config.h>
 
 #include <CGAL/Bbox_3.h>
 
 #include <CGAL/point_generators_3.h>
-#include <CGAL/Mesh_3/Creator_weighted_point_3.h>
 
 #include <boost/variant.hpp>
 #include <boost/format.hpp>
@@ -721,4 +722,6 @@ Labeled_mesh_domain_3<F,BGT,Null>::Construct_initial_points::operator()(
 
 }  // end namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // LABELLED_MESH_TRAITS_3_H_
