@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: GPL-3.0+
 //
 //
 // Author(s)     : Sebastien Loriot and Andreas Fabri
@@ -23,6 +24,7 @@
 
 #include <CGAL/license/Polygon_mesh_processing/connected_components.h>
 
+#include <CGAL/disable_warnings.h>
 
 #include<set>
 #include<vector>
@@ -106,13 +108,13 @@ namespace Polygon_mesh_processing{
  *  \tparam FaceOutputIterator a model of `OutputIterator` that accepts
         faces of type
         `boost::graph_traits<PolygonMesh>::%face_descriptor`.
- *  \tparam NamedParameters a sequence of \ref namedparameters
+ *  \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
  *
  *  \param seed_face a face of `pmesh` from which exploration starts to detect the connected component
            that contains it
  *  \param pmesh the polygon mesh
  *  \param out the output iterator that collects faces from the same connected component as `seed_face`
- *  \param np optional \ref namedparameters described below
+ *  \param np optional \ref pmp_namedparameters "Named Parameters" described below
  *
  * \cgalNamedParamsBegin
  *    \cgalParamBegin{edge_is_constrained_map}  a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
@@ -135,12 +137,12 @@ connected_component(typename boost::graph_traits<PolygonMesh>::face_descriptor s
   using boost::get_param;
 
   typedef typename boost::lookup_named_param_def <
-    CGAL::edge_is_constrained_t,
+    internal_np::edge_is_constrained_t,
     NamedParameters,
     internal::No_constraint<PolygonMesh>//default
   > ::type                                               EdgeConstraintMap;
   EdgeConstraintMap ecmap
-    = choose_param(get_param(np, edge_is_constrained),
+    = choose_param(get_param(np, internal_np::edge_is_constrained),
                    internal::No_constraint<PolygonMesh>());
 
   typedef typename boost::graph_traits<PolygonMesh>::face_descriptor face_descriptor;
@@ -182,17 +184,17 @@ connected_component(typename boost::graph_traits<PolygonMesh>::face_descriptor s
  *  computes for each face the index of the corresponding connected component.
  *
  *  A property map for `CGAL::face_index_t` should be either available as an internal property map 
- *  to `pmesh` or provided as one of the \ref namedparameters.
+ *  to `pmesh` or provided as one of the \ref pmp_namedparameters "Named Parameters".
  *
  *  \tparam PolygonMesh a model of `FaceListGraph`
  *  \tparam FaceComponentMap a model of `WritablePropertyMap` with
         `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and
         `boost::graph_traits<PolygonMesh>::%faces_size_type` as value type.
- *  \tparam NamedParameters a sequence of \ref namedparameters
+ *  \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
 
  * \param pmesh the polygon mesh
  * \param fcm the property map with indices of components associated to faces in `pmesh`
- * \param np optional \ref namedparameters described below
+ * \param np optional \ref pmp_namedparameters "Named Parameters" described below
  *
  * \cgalNamedParamsBegin
  *    \cgalParamBegin{edge_is_constrained_map} a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
@@ -215,12 +217,12 @@ connected_components(const PolygonMesh& pmesh,
   using boost::get_param;
 
   typedef typename boost::lookup_named_param_def <
-    CGAL::edge_is_constrained_t,
+    internal_np::edge_is_constrained_t,
     NamedParameters,
     internal::No_constraint<PolygonMesh>//default
   > ::type                                               EdgeConstraintMap;
   EdgeConstraintMap ecmap
-    = choose_param(get_param(np, edge_is_constrained),
+    = choose_param(get_param(np, internal_np::edge_is_constrained),
                    internal::No_constraint<PolygonMesh>());
 
   typedef Dual<PolygonMesh>                              Dual;
@@ -232,8 +234,8 @@ connected_components(const PolygonMesh& pmesh,
     internal::No_border<PolygonMesh, EdgeConstraintMap>(pmesh, ecmap));
 
   typename GetFaceIndexMap<PolygonMesh, NamedParameters>::const_type
-    fimap = choose_param(get_param(np, face_index),
-                         get_const_property_map(face_index, pmesh));
+    fimap = choose_param(get_param(np, internal_np::face_index),
+                         get_const_property_map(boost::face_index, pmesh));
 
   return boost::connected_components(finite_dual,
     fcm,
@@ -268,14 +270,14 @@ void keep_connected_components(PolygonMesh& pmesh
  *
  * Property maps for `CGAL::face_index_t` and `CGAL::vertex_index_t`
  * should be either available as internal property maps 
- * to `pmesh` or provided as \ref namedparameters.
+ * to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
  *
  * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
- * \tparam NamedParameters a sequence of \ref namedparameters
+ * \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
  *
  * \param pmesh the polygon mesh
  * \param nb_components_to_keep the number of components to be kept
- * \param np optional \ref namedparameters described below
+ * \param np optional \ref pmp_namedparameters "Named Parameters" described below
  *
  * \cgalNamedParamsBegin
  *    \cgalParamBegin{edge_is_constrained_map} a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
@@ -299,7 +301,7 @@ std::size_t keep_largest_connected_components(PolygonMesh& pmesh
 
   //FaceIndexMap
   typedef typename GetFaceIndexMap<PM, NamedParameters>::type FaceIndexMap;
-  FaceIndexMap fimap = choose_param(get_param(np, boost::face_index),
+  FaceIndexMap fimap = choose_param(get_param(np, internal_np::face_index),
                                     get_property_map(boost::face_index, pmesh));
 
   //vector_property_map
@@ -351,14 +353,14 @@ std::size_t keep_largest_connected_components(PolygonMesh& pmesh,
  *
  * Property maps for `CGAL::face_index_t` and `CGAL::vertex_index_t`
  * should be either available as internal property maps 
- * to `pmesh` or provided as \ref namedparameters.
+ * to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
  *
  * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
- * \tparam NamedParameters a sequence of \ref namedparameters
+ * \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
  *
  * \param pmesh the polygon mesh
  * \param threshold_components_to_keep the number of faces a component must have so that it is kept
- * \param np optional \ref namedparameters described below
+ * \param np optional \ref pmp_namedparameters "Named Parameters" described below
  *
  * \cgalNamedParamsBegin
  *    \cgalParamBegin{edge_is_constrained_map} a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
@@ -382,7 +384,7 @@ std::size_t keep_large_connected_components(PolygonMesh& pmesh
 
   //FaceIndexMap
   typedef typename GetFaceIndexMap<PM, NamedParameters>::type FaceIndexMap;
-  FaceIndexMap fim = choose_param(get_param(np, boost::face_index),
+  FaceIndexMap fim = choose_param(get_param(np, internal_np::face_index),
                                   get_property_map(boost::face_index, pmesh));
 
   //vector_property_map
@@ -444,7 +446,7 @@ void keep_or_remove_connected_components(PolygonMesh& pmesh
 
   //VertexIndexMap
   typedef typename GetVertexIndexMap<PM, NamedParameters>::type VertexIndexMap;
-  VertexIndexMap vim = choose_param(get_param(np, boost::vertex_index),
+  VertexIndexMap vim = choose_param(get_param(np, internal_np::vertex_index),
                                     get_const_property_map(boost::vertex_index, pmesh));
 
   std::set<std::size_t> cc_to_keep;
@@ -566,10 +568,10 @@ void keep_or_remove_connected_components(PolygonMesh& pmesh
 *
 * Property maps for `CGAL::vertex_index_t`
 * should be either available as internal property map
-* to `pmesh` or provided as \ref namedparameters.
+* to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
 *
 * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-* \tparam NamedParameters a sequence of \ref namedparameters
+* \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
 * \tparam ComponentRange a range of ids convertible to `std::size`
 * \tparam FaceComponentMap a model of `ReadWritePropertyMap` with
 *         `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and
@@ -579,7 +581,7 @@ void keep_or_remove_connected_components(PolygonMesh& pmesh
 * \param pmesh the polygon mesh
 * \param fcm the property map with indices of components associated to faces in `pmesh`.
 *        After calling this function, the values of `fcm` are undefined.
-* \param np optional \ref namedparameters described below
+* \param np optional \ref pmp_namedparameters "Named Parameters" described below
 *
 * \cgalNamedParamsBegin
 *    \cgalParamBegin{vertex_index_map} a property map containing the index of each vertex of `pmesh` \cgalParamEnd
@@ -609,11 +611,11 @@ void keep_connected_components(PolygonMesh& pmesh
 *
 * Property maps for `CGAL::vertex_index_t`
 * should be either available as internal property map
-* to `pmesh` or provided as \ref namedparameters.
+* to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
 *
 *
 * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-* \tparam NamedParameters a sequence of \ref namedparameters
+* \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
 * \tparam ComponentRange a range of ids convertible to `std::size`
 * \tparam FaceComponentMap a model of `ReadWritePropertyMap` with
 *         `boost::graph_traits<PolygonMesh>::%face_descriptor` as key type and
@@ -623,7 +625,7 @@ void keep_connected_components(PolygonMesh& pmesh
 * \param pmesh the polygon mesh
 * \param fcm the property map with indices of components associated to faces in `pmesh`.
 *        After calling this function, the values of `fcm` are undefined.
-* \param np optional \ref namedparameters described below
+* \param np optional \ref pmp_namedparameters "Named Parameters" described below
 *
 * \cgalNamedParamsBegin
 *    \cgalParamBegin{vertex_index_map} a property map containing the index of each vertex of `pmesh` \cgalParamEnd
@@ -650,19 +652,19 @@ void remove_connected_components(PolygonMesh& pmesh
 *
 * Property maps for `CGAL::face_index_t` and `CGAL::vertex_index_t`
 * should be either available as internal property maps
-* to `pmesh` or provided as \ref namedparameters.
+* to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
 *
 * \note If the removal of the connected components makes `pmesh` a non-manifold surface,
 * then the behavior of this function is undefined.
 *
 * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-* \tparam NamedParameters a sequence of \ref namedparameters
+* \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
 * \tparam FaceRange a range of `boost::graph_traits<PolygonMesh>::%face_descriptor`
 *         indicating the connected components to be removed.
 *
 * \param components_to_remove a face range, including one face or more on each component to be removed
 * \param pmesh the polygon mesh
-* \param np optional \ref namedparameters described below
+* \param np optional \ref pmp_namedparameters "Named Parameters" described below
 *
 * \cgalNamedParamsBegin
 *    \cgalParamBegin{edge_is_constrained_map} a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
@@ -686,7 +688,7 @@ void remove_connected_components(PolygonMesh& pmesh
 
   //FaceIndexMap
   typedef typename GetFaceIndexMap<PM, CGAL_PMP_NP_CLASS>::type FaceIndexMap;
-  FaceIndexMap fim = choose_param(get_param(np, boost::face_index),
+  FaceIndexMap fim = choose_param(get_param(np, internal_np::face_index),
                                   get_property_map(boost::face_index, pmesh));
 
   //vector_property_map
@@ -708,19 +710,19 @@ void remove_connected_components(PolygonMesh& pmesh
 *
 * Property maps for `CGAL::face_index_t` and `CGAL::vertex_index_t`
 * should be either available as internal property maps
-* to `pmesh` or provided as \ref namedparameters.
+* to `pmesh` or provided as \ref pmp_namedparameters "Named Parameters".
 *
 * \note If the removal of the connected components makes `pmesh` a non-manifold surface,
 * then the behavior of this function is undefined.
 *
 * \tparam PolygonMesh a model of `FaceListGraph` and `MutableFaceGraph`
-* \tparam NamedParameters a sequence of \ref namedparameters
+* \tparam NamedParameters a sequence of \ref pmp_namedparameters "Named Parameters"
 * \tparam FaceRange a range of `boost::graph_traits<PolygonMesh>::%face_descriptor`
 *         indicating the connected components to be kept.
 *
 * \param pmesh the polygon mesh
 * \param components_to_keep a face range, including one face or more on each component to be kept
-* \param np optional \ref namedparameters described below
+* \param np optional \ref pmp_namedparameters "Named Parameters" described below
 *
 * \cgalNamedParamsBegin
 *    \cgalParamBegin{edge_is_constrained_map} a property map containing the constrained-or-not status of each edge of `pmesh` \cgalParamEnd
@@ -744,7 +746,7 @@ void keep_connected_components(PolygonMesh& pmesh
 
   //FaceIndexMap
   typedef typename GetFaceIndexMap<PM, CGAL_PMP_NP_CLASS>::type FaceIndexMap;
-  FaceIndexMap fim = choose_param(get_param(np, boost::face_index),
+  FaceIndexMap fim = choose_param(get_param(np, internal_np::face_index),
                                   get_property_map(boost::face_index, pmesh));
 
   //vector_property_map
@@ -803,4 +805,6 @@ void keep_connected_components(PolygonMesh& pmesh
 
 } // namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif //CGAL_POLYGON_MESH_PROCESSING_CONNECTED_COMPONENTS_H
